@@ -23,7 +23,6 @@ BEGIN {
       constant->import( MAKE => $make );
       last;
    }
-   die "OS unsupported - unable to find GNU make\n" unless defined &MAKE;
 
    # GNU libtool is called 'glibtool' on Darwin
    foreach my $libtool (qw( libtool glibtool )) {
@@ -35,14 +34,157 @@ BEGIN {
       constant->import( LIBTOOL => $libtool );
       last;
    }
-   die "OS unsupported - unable to find GNU libtool\n" unless defined &LIBTOOL;
-
 }
 
-use constant MAKEARGS => ( ( LIBTOOL eq "libtool" ) ? ( "LIBTOOL=".LIBTOOL ) : () );
+sub MAKEARGS { "LIBTOOL=".LIBTOOL() }
 
 __PACKAGE__->add_property( 'tarball' );
 __PACKAGE__->add_property( 'pkgconfig_module' );
+__PACKAGE__->add_property( 'pkgconfig_version' );
+__PACKAGE__->add_property( 'alien_requires' );
+
+# Modules that this code itself requires
+my %more_configure_requires = (
+   'File::Basename' => 0,
+   'File::Spec'     => 0,
+   'File::Path'     => '2.07',
+   'Module::Build'  => 0,
+);
+
+sub new
+{
+   my $class = shift;
+   my %args = @_;
+
+   my $use_bundled = 0;
+
+   $args{get_options}{bundled} = {
+      store => \$use_bundled,
+      type  => "+",
+   };
+
+
+   my $self = $class->SUPER::new( %args );
+
+   foreach my $req ( @{ $self->alien_requires || [] } ) {
+      my $missing = $self->do_requires( @$req );
+      die "OS unsupported - missing $missing\n" if defined $missing;
+   }
+
+   my @reqs = @{ $self->alien_requires || [] };
+   while( @reqs ) {
+      my ( $name, @args ) = @{ shift @reqs };
+
+      push( @reqs, @args ), next if $name eq "any";
+
+      $self->configure_requires->{"ExtUtils::CChecker"} //= 0 if $name eq "header";
+   }
+
+   $self->configure_requires->{$_} ||= $more_configure_requires{$_} for keys %more_configure_requires;
+
+   my $module = $self->pkgconfig_module;
+   my $version = $self->pkgconfig_version;
+
+   $use_bundled = 1 if
+      !$use_bundled and system( "pkg-config", $module, "--atleast-version", $version ) != 0;
+
+   if( $use_bundled ) {
+      die "OS unsupported - unable to find GNU make\n" unless defined &MAKE;
+      die "OS unsupported - unable to find GNU libtool\n" unless defined &LIBTOOL;
+      print "Building bundled source\n";
+   }
+   else {
+      print "Detected $module version >= $version from pkg-config\n";
+   }
+
+   $self->notes( use_bundled => $use_bundled );
+
+   return $self;
+}
+
+{
+   my $eucc;
+
+   sub cchecker
+   {
+      my $self = shift;
+      return $eucc ||= do {
+         require ExtUtils::CChecker;
+         return ExtUtils::CChecker->new;
+      };
+   }
+}
+
+sub do_requires
+{
+   my $self = shift;
+   my ( $name, @args ) = @_;
+
+   my $code = $self->can( "do_requires_$name" ) or
+      die "Unrecognised 'alien_requires' requirement type '$name'\n";
+
+   return $self->$code( @args );
+}
+
+sub do_requires_any
+{
+   my $self = shift;
+   my @alts = @_;
+
+   my @missing;
+   foreach my $alt ( @alts ) {
+      my $ret = $self->do_requires( @$alt );
+      return if !$ret;
+
+      push @missing, $ret;
+   }
+
+   return $missing[0] if @missing < 2;
+   return "either $missing[0] or $missing[1]" if @missing == 2;
+   return "any of " . join( ", ", @missing[0 .. $#missing-1] ) . " or " . $missing[-1];
+}
+
+sub do_requires_pkgconfig
+{
+   my $self = shift;
+   my ( $module, %args ) = @_;
+
+   print "Looking for pkg-config $module... ";
+
+   my @cmdline;
+   push @cmdline, "--atleast-version=$args{atleast_version}" if defined $args{atleast_version};
+
+   if( system( "pkg-config", $module, @cmdline ) == 0 ) {
+      print "found\n";
+      return;
+   }
+
+   print "not found\n";
+   return "$module";
+}
+
+sub do_requires_header
+{
+   my $self = shift;
+   my ( $header ) = @_;
+
+   print "Looking for <$header>... ";
+
+   my $success = $self->cchecker->try_compile_run(
+      source => <<"EOC"
+#include <$header>
+int main(int argc, char *argv[]) {return 0;}
+EOC
+   );
+
+   if( $success ) {
+      print "found\n";
+      return;
+   }
+
+   print "not found\n";
+   return "<$header>";
+}
 
 sub _srcdir
 {
@@ -74,7 +216,7 @@ sub make_in_srcdir
    my @args = @_;
 
    $self->in_srcdir( sub {
-      system( MAKE, MAKEARGS, @args ) == 0 or
+      system( MAKE(), MAKEARGS(), @args ) == 0 or
          die "Unable to make - $!";
    } );
 }
@@ -83,6 +225,8 @@ sub ACTION_src
 {
    my $self = shift;
 
+   return unless $self->notes( 'use_bundled' );
+
    -d $self->_srcdir and return;
 
    my $tarball = $self->tarball;
@@ -109,11 +253,14 @@ sub ACTION_code
    my $incdir = File::Spec->catdir( $libdir, "include" );
    my $mandir = File::Spec->catdir( $blib, "libdoc" );
 
+   # All these at least must exist
+   -d $_ or mkdir $_ for $blib, $libdir;
+
    my $pkgconfig_module = $self->pkgconfig_module;
 
    my $buildstamp = $self->_stampfile( "build" );
 
-   unless( -f $buildstamp ) {
+   if( $self->notes( 'use_bundled' ) and !-f $buildstamp ) {
       $self->depends_on( 'src' );
 
       $self->make_in_srcdir( () );
@@ -151,8 +298,8 @@ sub ACTION_code
    my $dstfile = File::Spec->catfile( $blib, "lib", @module_file );
 
    unless( $self->up_to_date( $srcfile, $dstfile ) ) {
-
       my %replace = (
+         USE_BUNDLED      => $self->notes( 'use_bundled' ),
          PKGCONFIG_MODULE => $pkgconfig_module,
       );
 
@@ -176,7 +323,7 @@ sub cp_file_with_replacement
    my $dstfile = $args{dstfile};
    my $replace = $args{replace};
 
-   make_path( dirname( $dstfile ), 0, 0777 );
+   make_path( dirname( $dstfile ), { mode => 0777 } );
 
    open( my $inh,  "<", $srcfile ) or die "Cannot read $srcfile - $!";
    open( my $outh, ">", $dstfile ) or die "Cannot write $dstfile - $!";
@@ -191,6 +338,8 @@ sub ACTION_test
 {
    my $self = shift;
 
+   return unless $self->notes( 'use_bundled' );
+
    $self->depends_on( "code" );
 
    $self->make_in_srcdir( "test" );
@@ -200,11 +349,13 @@ sub ACTION_clean
 {
    my $self = shift;
 
-   if( -d $self->_srcdir ) {
-      $self->make_in_srcdir( "clean" );
-   }
+   if( $self->notes( 'use_bundled' ) ) {
+      if( -d $self->_srcdir ) {
+         $self->make_in_srcdir( "clean" );
+      }
 
-   unlink( $self->_stampfile( "build" ) );
+      unlink( $self->_stampfile( "build" ) );
+   }
 
    $self->SUPER::ACTION_clean;
 }
